@@ -1,11 +1,12 @@
 #
 # = bio/appl/blast/format0.rb - BLAST default output (-m 0) parser
 # 
-# Author:: Naohisa GOTO
-# Copyright:: Copyright (C) 2003-2006 GOTO Naohisa <ng@bioruby.org>
-# License:: Ruby's
+# Copyright::  Copyright (C) 2003-2006 GOTO Naohisa <ng@bioruby.org>
+# License::    The Ruby License
 #
-#  $Id: format0.rb,v 1.17 2006/02/22 08:46:15 ngoto Exp $
+# $Id: format0.rb,v 1.22 2007/04/21 08:58:17 ngoto Exp $
+#
+# == Description
 #
 # NCBI BLAST default (-m 0 option) output parser.
 #
@@ -326,7 +327,9 @@ module Bio
               sc.skip(/\s*/)
               while sc.rest?
                 if sc.match?(/Number of sequences better than +([e\-\.\d]+) *\: *(.+)/) then
-                  @expect = sc[1]
+                  ev = sc[1]
+                  ev = '1' + ev if ev[0] == ?e
+                  @expect = ev.to_f
                   @num_hits = sc[2].tr(',', '').to_i
                 end
                 if sc.skip(/([\-\,\.\'\(\)\#\w ]+)\: *(.*)/) then
@@ -656,8 +659,11 @@ module Bio
                     z.each { |y| y.reverse! }
                     h = Hit.new([ z.pop.to_s.sub(/\.+\z/, '') ])
                     bs = z.pop.to_s
+                    bs = '1' + bs if bs[0] == ?e
+                    bs = (bs.empty? ? nil : bs.to_f)
                     ev = z.pop.to_s
-                    #ev = '1' + ev if ev[0] == ?e
+                    ev = '1' + ev if ev[0] == ?e
+                    ev = (ev.empty? ? (1.0/0.0) : ev.to_f)
                     h.instance_eval { @bit_score = bs; @evalue = ev }
                     @hits << h
                   end
@@ -780,7 +786,7 @@ module Bio
           def initialize(data)
             @f0hitname = data.shift
             @hsps = []
-            while r = data[0] and /^\s+Score/ =~ r
+            while r = data[0] and /\A\s+Score/ =~ r
               @hsps << HSP.new(data)
             end
             @again = false
@@ -948,11 +954,14 @@ module Bio
               while sc.rest?
                 sc.skip(/\s*/)
                 if sc.skip(/Expect(?:\(\d\))? *\= *([e\-\.\d]+)/) then
-                  @evalue = sc[1]
-                  #@evalue = '1' + @evalue if @evalue[0] == ?e
+                  ev = sc[1].to_s
+                  ev = '1' + ev if ev[0] == ?e
+                  @evalue = ev.to_f
                 elsif sc.skip(/Score *\= *([e\-\.\d]+) *bits *\( *([e\-\.\d]+) *\)/) then
-                  @bit_score = sc[1]
-                  @score = sc[2]
+                  bs = sc[1]
+                  bs = '1' + bs if bs[0] == ?e
+                  @bit_score = bs.to_f
+                  @score = sc[2].to_i
                 elsif sc.skip(/(Identities|Positives|Gaps) *\= (\d+) *\/ *(\d+) *\(([\.\d]+) *\% *\)/) then
                   alen = sc[3].to_i
                   @align_len = alen unless defined?(@align_len)
@@ -960,13 +969,13 @@ module Bio
                   case sc[1]
                   when 'Identities'
                     @identity = sc[2].to_i
-                    @percent_identity = sc[4]
+                    @percent_identity = sc[4].to_i
                   when 'Positives'
                     @positive = sc[2].to_i
-                    @percent_positive = sc[4]
+                    @percent_positive = sc[4].to_i
                   when 'Gaps'
                     @gaps = sc[2].to_i
-                    @percent_gaps = sc[4]
+                    @percent_gaps = sc[4].to_i
                   else
                     raise ScanError
                   end
@@ -994,16 +1003,22 @@ module Bio
                   end
                 elsif sc.skip(/Score *\= *([e\-\.\d]+) +\(([e\-\.\d]+) *bits *\)/) then
                   #WU-BLAST
-                  @score = sc[1]
-                  @bit_score = sc[2]
+                  @score = sc[1].to_i
+                  bs = sc[2]
+                  bs = '1' + bs if bs[0] == ?e
+                  @bit_score = bs.to_f
                 elsif sc.skip(/P *\= * ([e\-\.\d]+)/) then
                   #WU-BLAST
                   @p_sum_n = nil
-                  @pvalue = sc[1]
+                  pv = sc[1]
+                  pv = '1' + pv if pv[0] == ?e
+                  @pvalue = pv.to_f
                 elsif sc.skip(/Sum +P *\( *(\d+) *\) *\= *([e\-\.\d]+)/) then
                   #WU-BLAST
                   @p_sum_n = sc[1].to_i
-                  @pvalue = sc[2]
+                  pv = sc[2]
+                  pv = '1' + pv if pv[0] == ?e
+                  @pvalue = pv.to_f
                 else
                   raise ScanError
                 end
