@@ -1,19 +1,22 @@
 #
 # = bio/appl/blast/wublast.rb - WU-BLAST default output parser
 # 
-# Copyright:: Copyright (C) 2003 GOTO Naohisa <ng@bioruby.org>
-# License:: Ruby's
+# Copyright::  Copyright (C) 2003 GOTO Naohisa <ng@bioruby.org>
+# License::    The Ruby License
 #
-#  $Id: wublast.rb,v 1.6 2006/02/22 08:46:15 ngoto Exp $
+# $Id: wublast.rb,v 1.11 2007/04/21 08:58:17 ngoto Exp $
 #
-#  WU-BLAST default output parser.
+# == Description
 #
-#  The parser is still incomplete and may contain many bugs,
-#  because I didn't have WU-BLAST license.
-#  It was tested under web-based WU-BLAST results and
-#  obsolete version downloaded from http://blast.wustl.edu/ .
+# WU-BLAST default output parser.
+#
+# The parser is still incomplete and may contain many bugs,
+# because I didn't have WU-BLAST license.
+# It was tested under web-based WU-BLAST results and
+# obsolete version downloaded from http://blast.wustl.edu/ .
+#
+# == References
 #
-# = References
 # * http://blast.wustl.edu/
 # * http://www.ebi.ac.uk/blast2/
 #
@@ -134,6 +137,10 @@ module Bio
                 end
               end
             end
+            if ev = @parameters['E'] then
+              ev = '1' + ev if ev[0] == ?e
+              @parameters['E'] = ev.to_f
+            end
             @parse_parameters = true
           end
         end
@@ -245,10 +252,12 @@ module Bio
                     @hits[i] = h
                   end
                   z.pop if flag_tblast #ignore Reading Frame
-                  scr = z.pop.to_s
+                  scr = z.pop
+                  scr = (scr ? scr.to_i : nil)
                   pval = z.pop.to_s
+                  pval = '1' + pval if pval[0] == ?e
+                  pval = (pval.empty? ? (1.0/0.0) : pval.to_f)
                   nnum = z.pop.to_i
-                  #ev = '1' + ev if ev[0] == ?e
                   h.instance_eval {
                     @score = scr
                     @pvalue = pval
@@ -280,7 +289,7 @@ module Bio
                 data.shift
                 r = data.first
               end
-              if /^\s+Score/ =~ r then
+              if /\A\s+Score/ =~ r then
                 @hsps << HSP.new(data)
               else
                 break
