@@ -2,25 +2,9 @@
 # = bio/appl/blat/report.rb - BLAT result parser
 #
 # Copyright:: Copyright (C) 2004 GOTO Naohisa <ng@bioruby.org>
-# License:: LGPL
+# License:: The Ruby License
 #
-#--
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Lesser General Public
-#  License as published by the Free Software Foundation; either
-#  version 2 of the License, or (at your option) any later version.
-#
-#  This library is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Lesser General Public License for more details.
-#
-#  You should have received a copy of the GNU Lesser General Public
-#  License along with this library; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-#++
-#
-#  $Id: report.rb,v 1.6 2005/12/18 15:58:39 k Exp $
+#  $Id: report.rb,v 1.13 2007/04/05 23:35:39 trevor Exp $
 #
 # BLAT result parser (psl / pslx format).
 #
@@ -36,7 +20,7 @@
 # * Kent, W.J., BLAT--the BLAST-like alignment tool,
 #   Genome Research, 12, 656--664, 2002.
 #   http://www.genome.org/cgi/content/abstract/12/4/656
-# 
+# * http://genome.ucsc.edu/goldenPath/help/blatSpec.html
 
 require 'bio'
 
@@ -49,7 +33,8 @@ module Bio
     # In BLAT results, the start position of a sequnece is numbered as 0.
     # On the other hand, in many other homology search programs,
     # the start position of a sequence is numbered as 1.
-    # To keep compatibility, the BLAT parser adds 1 to every position number.
+    # To keep compatibility, the BLAT parser adds 1 to every position number
+    # except Bio::Blat::Report::Seqdesc and some Bio::Blat specific methods.
     #
     # Note that Bio::Blat::Report#query_def, #query_id, #query_len  methods
     # simply return first hit's query_*.
@@ -73,6 +58,11 @@ module Bio
           if flag then
             @hits << Hit.new(line)
           else
+            # for headerless data
+            if /^\d/ =~ line then
+              flag = true
+              redo
+            end
             line = line.chomp
             if /\A\-+\s*\z/ =~ line
               flag = true
@@ -95,7 +85,15 @@ module Bio
 
       # Parses headers.
       def parse_header(ary)
-        ary.shift # first line is removed
+        while x = ary.shift
+          if /psLayout version (\S+)/ =~ x then
+            @psl_version = $1
+            break
+          elsif !(x.strip.empty?)
+            ary.unshift(x)
+            break
+          end
+        end
         a0 = ary.collect { |x| x.split(/\t/) }
         k = []
         a0.each do |x|
@@ -109,6 +107,9 @@ module Bio
       end
       private :parse_header
 
+      # version of the psl format (String or nil).
+      attr_reader :psl_version
+
       # Bio::Blat::Report::SeqDesc stores sequence information of
       # query or subject of the BLAT report.
       # It also includes some hit information.
@@ -154,12 +155,14 @@ module Bio
         # Creates a new SegmentPair object.
         # It is designed to be called internally from Bio::Blat::Report class.
         # Users shall not use it directly.
-        def initialize(query_len, strand,
-                       blksize, qstart, tstart, qseq, tseq)
+        def initialize(query_len, target_len, strand,
+                       blksize, qstart, tstart, qseq, tseq,
+                       protein_flag)
           @blocksize  = blksize
           @qseq       = qseq
           @hseq       = hseq
           @hit_strand   = 'plus'
+          w = (protein_flag ? 3 : 1) # 3 means query=protein target=dna
           case strand
           when '-'
             # query is minus strand
@@ -170,15 +173,26 @@ module Bio
             # To keep compatibility, with other homology search programs,
             # we add 1 to each position number.
             @hit_from   = tstart + 1
-            @hit_to     = tstart + blksize # - 1 + 1
-          else #when '+'
+            @hit_to     = tstart + blksize * w # - 1 + 1
+          when '+-'
+            # hit is minus strand
+            @query_strand = 'plus'
+            @hit_strand = 'minus'
+            # To keep compatibility, with other homology search programs,
+            # we add 1 to each position number.
+            @query_from   = qstart + 1
+            @query_to     = qstart + blksize # - 1 + 1
+            # convert positions
+            @hit_from     = target_len - tstart
+            @hit_to       = target_len - tstart - blksize * w + 1
+          else #when '+', '++'
             @query_strand = 'plus'
             # To keep compatibility with other homology search programs,
             # we add 1 to each position number.
             @query_from = qstart + 1
             @query_to   = qstart + blksize # - 1 + 1
             @hit_from   = tstart + 1
-            @hit_to     = tstart + blksize # - 1 + 1
+            @hit_to     = tstart + blksize * w # - 1 + 1
           end
         end
         # Returns query start position.
@@ -277,9 +291,13 @@ module Bio
         def match;       @data[0].to_i;  end
         # Mismatch nucleotides.
         def mismatch;    @data[1].to_i;  end
-        # rep. match (???)
+
+        # "rep. match".
+        # Number of bases that match but are part of repeats.
+        # Note that current version of BLAT always set 0.
         def rep_match;   @data[2].to_i;  end
-        # N's (???)
+
+        # "N's". Number of 'N' bases.
         def n_s;         @data[3].to_i;  end
 
         # Returns strand information of the hit.
@@ -308,9 +326,11 @@ module Bio
             tst   = target.starts
             qseqs = query.seqs
             tseqs = target.seqs
+            pflag = self.protein?
             @blocks = (0...block_count).collect do |i|
-              SegmentPair.new(query.size, strand, bs[i],
-                              qst[i], tst[i], qseqs[i], tseqs[i])
+              SegmentPair.new(query.size, target.size, strand, bs[i],
+                              qst[i], tst[i], qseqs[i], tseqs[i],
+                              pflag)
             end
           end
           @blocks
@@ -343,6 +363,83 @@ module Bio
         def each(&x) #:yields: segmentpair
           exons.each(&x)
         end
+
+        #--
+        # methods described in the BLAT FAQ at the UCSC genome browser.
+        # (http://genome.ucsc.edu/FAQ/FAQblat#blat4)
+        #++
+
+        # Calculates the pslCalcMilliBad value defined in the
+        # BLAT FAQ (http://genome.ucsc.edu/FAQ/FAQblat#blat4).
+        #
+        # The algorithm is taken from the BLAT FAQ
+        # (http://genome.ucsc.edu/FAQ/FAQblat#blat4).
+        def milli_bad
+          w = (self.protein? ? 3 : 1)
+          qalen = w * (self.query.end - self.query.start)
+          talen = self.target.end - self.target.start
+          alen = (if qalen < talen then qalen; else talen; end)
+          return 0 if alen <= 0
+          d = qalen - talen
+          d = 0 if d < 0
+          total = w * (self.match + self.rep_match + self.mismatch)
+          return 0 if total == 0
+          return (1000 * (self.mismatch * w + self.query.gap_count +
+                            (3 * Math.log(1 + d)).round) / total)
+        end
+
+        # Calculates the percent identity compatible with the BLAT web server
+        # as described in the BLAT FAQ
+        # (http://genome.ucsc.edu/FAQ/FAQblat#blat4).
+        #
+        # The algorithm is taken from the BLAT FAQ
+        # (http://genome.ucsc.edu/FAQ/FAQblat#blat4).
+        def percent_identity
+          100.0 - self.milli_bad * 0.1
+        end
+
+        # When the output data comes from the protein query, returns true.
+        # Otherwise (nucleotide query), returns false.
+        # It returns nil if this cannot be determined.
+        #
+        # The algorithm is taken from the BLAT FAQ
+        # (http://genome.ucsc.edu/FAQ/FAQblat#blat4).
+        #
+        # Note: It seems that it returns true only when protein query
+        # with nucleotide database (blat options: -q=prot -t=dnax).
+        def protein?
+          return nil if self.block_sizes.empty?
+          case self.strand[1,1]
+          when '+'
+            if self.target.end == self.target.starts[-1] +
+                3 * self.block_sizes[-1] then
+              true
+            else
+              false
+            end
+          when '-'
+            if self.target.start == self.target.size -
+                self.target.starts[-1] - 3 * self.block_sizes[-1] then
+              true
+            else
+              false
+            end
+          else
+            nil
+          end
+        end
+
+        # Calculates the score compatible with the BLAT web server
+        # as described in the BLAT FAQ
+        # (http://genome.ucsc.edu/FAQ/FAQblat#blat4).
+        #
+        # The algorithm is taken from the BLAT FAQ
+        # (http://genome.ucsc.edu/FAQ/FAQblat#blat4).
+        def score
+          w = (self.protein? ? 3 : 1)
+          w * (self.match + (self.rep_match >> 1)) -
+            w * self.mismatch - self.query.gap_count - self.target.gap_count
+        end
       end #class Hit
 
       #--
