@@ -1,38 +1,22 @@
 #
 # = bio/appl/psort/report.rb - PSORT systems report classes
 #
-# Copyright::   Copyright (C) 2003 Mitsuteru C. Nakao <n@bioruby.org>
-# License::     LGPL
+# Copyright::   Copyright (C) 2003 
+#               Mitsuteru C. Nakao <n@bioruby.org>
+# License::     The Ruby License
 #
-#  $Id: report.rb,v 1.12 2005/11/03 10:50:58 nakao Exp $
+#  $Id: report.rb,v 1.15 2007/04/05 23:35:40 trevor Exp $
 #
 # == A Report classes for PSORT Systems
 # 
-#--
-#
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Lesser General Public
-#  License as published by the Free Software Foundation; either
-#  version 2 of the License, or (at your option) any later version.
-#
-#  This library is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Lesser General Public License for more details.
-#
-#  You should have received a copy of the GNU Lesser General Public
-#  License along with this library; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-#
-# ++
-#
 
-require 'bio/sequence'
 require 'bio/appl/psort'
 
 
 module Bio
 
+  autoload :Sequence, 'bio/sequence'
+
   class PSORT
 
     class PSORT1
