@@ -1,23 +1,10 @@
 #
 # bio/db/fantom.rb - RIKEN FANTOM2 database classes
 #
-#   Copyright (C) 2003 GOTO Naohisa <ngoto@gen-info.osaka-u.ac.jp>
+# Copyright:: Copyright (C) 2003 GOTO Naohisa <ng@bioruby.org> 
+# License::   The Ruby License
 #
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Lesser General Public
-#  License as published by the Free Software Foundation; either
-#  version 2 of the License, or (at your option) any later version.
-#
-#  This library is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Lesser General Public License for more details.
-#
-#  You should have received a copy of the GNU Lesser General Public
-#  License along with this library; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-#
-#  $Id: fantom.rb,v 1.11 2005/09/26 13:00:06 k Exp $
+#  $Id: fantom.rb,v 1.14 2007/04/05 23:35:40 trevor Exp $
 #
 
 begin
@@ -28,6 +15,7 @@ require 'uri'
 require 'net/http'
 
 require 'bio/db'
+require 'bio/command'
 #require 'bio/sequence'
 
 module Bio
@@ -45,11 +33,18 @@ module Bio
       addr = 'fantom.gsc.riken.go.jp'
       port = 80
       path = "/db/maxml/maxmlseq.cgi?masterid=#{URI.escape(idstr.to_s)}&style=xml"
-      proxy = URI.parse(http_proxy.to_s)
       xml = ''
-      Net::HTTP.start(addr, port, proxy.host, proxy.port) do |http|
-        response, = http.get(path)
-        xml = response.body
+      if http_proxy then
+        proxy = URI.parse(http_proxy.to_s)
+        Net::HTTP.start(addr, port, proxy.host, proxy.port) do |http|
+          response, = http.get(path)
+          xml = response.body
+        end
+      else
+        Bio::Command.start_http(addr, port) do |http|
+          response, = http.get(path)
+          xml = response.body
+        end
       end
       xml
     end
