@@ -4,25 +4,9 @@
 # Copyright::  Copyright (C) 2004, 2006
 #              Alex Gutteridge <alexg@ebi.ac.uk>
 #              Naohisa Goto <ng@bioruby.org>
-# License:: LGPL
+# License::    The Ruby License
 #
-#  $Id: residue.rb,v 1.10 2006/01/20 13:54:08 ngoto Exp $
-#
-#--
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Lesser General Public
-#  License as published by the Free Software Foundation; either
-#  version 2 of the License, or (at your option) any later version.
-#
-#  This library is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Lesser General Public License for more details.
-#
-#  You should have received a copy of the GNU Lesser General Public
-#  License along with this library; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-#++
+# $Id: residue.rb,v 1.13 2007/04/23 16:03:25 ngoto Exp $
 #
 # = Bio::PDB::Residue
 #
@@ -81,9 +65,9 @@ module Bio
       # Now, Residue#id is an alias of residue_id.
       alias id residue_id
 
-      #Keyed access to atoms based on element e.g. ["CA"]
+      #Keyed access to atoms based on atom name e.g. ["CA"]
       def [](key)
-        atom = @atoms.find{ |atom| key == atom.element }
+        atom = @atoms.find{ |atom| key == atom.name }
       end
 
       # Updates residue id. This is a private method.
