@@ -1,12 +1,12 @@
 #
 # = bio/io/das.rb - BioDAS access module
 #
-# Copyright::	Copyright (C) 2003, 2004
+# Copyright::	Copyright (C) 2003, 2004, 2007
 #		Shuichi Kawashima <shuichi@hgc.jp>,
 #		Toshiaki Katayama <k@bioruby.org>
-# License::	LGPL
+# License::	The Ruby License
 #
-# $Id: das.rb,v 1.11 2005/11/14 02:01:54 k Exp $
+# $Id: das.rb,v 1.17 2007/07/16 18:06:12 k Exp $
 #
 #--
 # == TODO
@@ -15,31 +15,12 @@
 #
 #++
 #
-#--
-#
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Lesser General Public
-#  License as published by the Free Software Foundation; either
-#  version 2 of the License, or (at your option) any later version.
-#
-#  This library is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Lesser General Public License for more details.
-#
-#  You should have received a copy of the GNU Lesser General Public
-#  License along with this library; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-#
-#++
-#
 
 begin
   require 'rexml/document'
 rescue LoadError
 end
-require 'uri'
-require 'net/http'
+require 'bio/command'
 require 'bio/sequence'
 
 
@@ -49,15 +30,24 @@ class DAS
 
   # Specify DAS server to connect
   def initialize(url = 'http://www.wormbase.org:80/db/')
-    schema, user, host, port, reg, path, = URI.split(url)
-    @server = Net::HTTP.new(host, port)
-    @prefix = path ? path.chomp('/') : ''
+    @server = url.chomp('/')
   end
 
+  def dna(dsn, entry_point, start, stop)
+    seg = Bio::DAS::SEGMENT.region(entry_point, start, stop)
+    self.get_dna(dsn, seg).first.sequence
+  end
+
+  def features(dsn, entry_point, start, stop)
+    seg = Bio::DAS::SEGMENT.region(entry_point, start, stop)
+    self.get_features(dsn, seg)
+  end
+
+
   # Returns an Array of Bio::DAS::DSN
   def get_dsn
     ary = []
-    result, = @server.get(@prefix + '/das/dsn')
+    result, = Bio::Command.post_form("#{@server}/das/dsn")
     doc = REXML::Document.new(result.body)
     doc.elements.each('/descendant::DSN') do |e|
       dsn = DSN.new
@@ -68,7 +58,7 @@ class DAS
           dsn.source_id = e.attributes['id']
           dsn.source_version = e.attributes['version']
         when 'MAPMASTER'
-          dsn.mapmaster = e.name
+          dsn.mapmaster = e.text
         when 'DESCRIPTION'
           dsn.description = e.text
           dsn.description_href = e.attributes['href']
@@ -81,10 +71,14 @@ class DAS
 
   # Returns Bio::DAS::ENTRY_POINT.
   # The 'dsn' can be a String or a Bio::DAS::DSN object.
-  def get_entry_point(dsn)
+  def get_entry_points(dsn)
     entry_point = ENTRY_POINT.new
-    dsn = dsn.source if dsn.instance_of?(Bio::DAS::DSN)
-    result, = @server.get(@prefix + '/das/' + dsn + '/entry_points')
+    if dsn.instance_of?(Bio::DAS::DSN)
+      src = dsn.source 
+    else
+      src = dsn
+    end
+    result, = Bio::Command.post_form("#{@server}/das/#{src}/entry_points")
     doc = REXML::Document.new(result.body)
     doc.elements.each('/descendant::ENTRY_POINTS') do |e|
       entry_point.href = e.attributes['href']
@@ -93,8 +87,8 @@ class DAS
         segment = SEGMENT.new
         segment.entry_id = e.attributes['id']
         segment.start = e.attributes['start']
-        segment.stop = e.attributes['stop']
-        segment.stop = e.attributes['orientation']
+        segment.stop = e.attributes['stop'] || e.attributes['size']
+        segment.orientation = e.attributes['orientation']
         segment.subparts = e.attributes['subparts']
         segment.description = e.text
         entry_point.segments << segment
@@ -117,9 +111,8 @@ class DAS
     segments.each do |s|
       opts << "segment=#{s.entry_id}:#{s.start},#{s.stop}"
     end
-    query = opts.join(';')
 
-    result, = @server.get(@prefix + '/das/' + dsn + '/dna?' + query)
+    result, = Bio::Command.post_form("#{@server}/das/#{dsn}/dna", opts)
     doc = REXML::Document.new(result.body)
     doc.elements.each('/descendant::SEQUENCE') do |e|
       sequence = DNA.new
@@ -150,9 +143,8 @@ class DAS
     segments.each do |s|
       opts << "segment=#{s.entry_id}:#{s.start},#{s.stop}"
     end
-    query = opts.join(';')
 
-    result, = @server.get(@prefix + '/das/' + dsn + '/sequence?' + query)
+    result, = Bio::Command.post_form("#{@server}/das/#{dsn}/sequence", opts)
     doc = REXML::Document.new(result.body)
     doc.elements.each('/descendant::SEQUENCE') do |e|
       sequence = SEQUENCE.new
@@ -188,9 +180,8 @@ class DAS
     segments.each do |s|
       opts << "segment=#{s.entry_id}:#{s.start},#{s.stop}"
     end
-    query = opts.join(';')
 
-    result, = @server.get(@prefix + '/das/' + dsn + '/types?' + query)
+    result, = Bio::Command.post_form("#{@server}/das/#{dsn}/types", opts)
     doc = REXML::Document.new(result.body)
     doc.elements.each('/descendant::GFF') do |e|
       types.version = e.attributes['version']
@@ -240,9 +231,8 @@ class DAS
     group_ids.each do |gid|
       opts << "group_id=#{gid}"
     end
-    query = opts.join(';')
 
-    result, = @server.get(@prefix + '/das/' + dsn + '/features?' + query)
+    result, = Bio::Command.post_form("#{@server}/das/#{dsn}/features", opts)
     doc = REXML::Document.new(result.body)
     doc.elements.each('/descendant::GFF') do |e|
       gff.version = e.attributes['version']
@@ -459,7 +449,7 @@ if __FILE__ == $0
   puts ">>> dsn : entry_points"
   org_list.each do |org|
     print "#{org} : "
-    list = kegg_das.get_entry_point(org)
+    list = kegg_das.get_entry_points(org)
     list.segments.each do |seg|
       print " #{seg.entry_id}"
     end
