@@ -2,28 +2,10 @@
 # = bio/io/ddbjxml.rb - DDBJ SOAP server access class
 #
 # Copyright::	Copyright (C) 2003, 2004
-#		KATAYAMA Toshiaki <k@bioruby.org>
-# License::	LGPL
+#		Toshiaki Katayama <k@bioruby.org>
+# License::	The Ruby License
 #
-# $Id: ddbjxml.rb,v 1.10 2006/02/02 16:30:29 nakao Exp $
-#
-#--
-#
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Lesser General Public
-#  License as published by the Free Software Foundation; either
-#  version 2 of the License, or (at your option) any later version.
-#
-#  This library is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Lesser General Public License for more details.
-#
-#  You should have received a copy of the GNU Lesser General Public
-#  License along with this library; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-#
-#++
+# $Id: ddbjxml.rb,v 1.14 2007/04/05 23:35:41 trevor Exp $
 #
 
 require 'bio/io/soapwsdl'
@@ -45,6 +27,13 @@ class XML < Bio::SOAPWSDL
 
   BASE_URI = "http://xml.nig.ac.jp/wsdl/"
 
+  # set default to GetEntry
+  SERVER_URI = BASE_URI + "GetEntry.wsdl"
+
+  def initialize(wsdl = nil)
+    super(wsdl || self.class::SERVER_URI)
+  end
+
   # === Description
   #
   # DDBJ XML BLAST Database Search 
@@ -72,11 +61,11 @@ class XML < Bio::SOAPWSDL
   # 
   # === WSDL Methods
   # 
-  # ==== searchSimple(program, database, query)
+  # * searchSimple(program, database, query)
   #
   # Returns a blast report in the default format.
   #
-  # ==== searchParam(program, database, query, param)
+  # * searchParam(program, database, query, param)
   #
   # Blasts with param and returns a blast report.
   #
@@ -86,11 +75,6 @@ class XML < Bio::SOAPWSDL
   #
   class Blast < XML
     SERVER_URI = BASE_URI + "Blast.wsdl"
-
-    # returns a Bio::DDBJ::XML::Blast object.
-    def initialize(wsdl = nil)
-      super(wsdl || SERVER_URI)
-    end
   end
 
 
@@ -123,8 +107,8 @@ class XML < Bio::SOAPWSDL
   #
   # === WSDL Methods
   #
-  # ==== analyzeSimple(query)
-  # ==== analyzeParam(query, param)
+  # * analyzeSimple(query)
+  # * analyzeParam(query, param)
   #
   # === References
   #
@@ -132,11 +116,6 @@ class XML < Bio::SOAPWSDL
   #
   class ClustalW < XML
     SERVER_URI = BASE_URI + "ClustalW.wsdl"
-
-    # returns a Bio::DDBJ::XML::ClustalW object.
-    def initialize(wsdl = nil)
-      super(wsdl || SERVER_URI)
-    end
   end
 
 
@@ -158,12 +137,12 @@ class XML < Bio::SOAPWSDL
   #
   # === WSDL Methods 
   #
-  # ==== getFFEntry(accession)
-  # ==== getXMLEntry(accession)
-  # ==== getFeatureInfo(accession, feature)
-  # ==== getAllFeatures(accession)
-  # ==== getRelatedFeatures(accession, start, stop)
-  # ==== getRelatedFeaturesSeq(accession, start, stop)
+  # * getFFEntry(accession)
+  # * getXMLEntry(accession)
+  # * getFeatureInfo(accession, feature)
+  # * getAllFeatures(accession)
+  # * getRelatedFeatures(accession, start, stop)
+  # * getRelatedFeaturesSeq(accession, start, stop)
   #
   # === References
   #
@@ -171,11 +150,6 @@ class XML < Bio::SOAPWSDL
   #
   class DDBJ < XML
     SERVER_URI = BASE_URI + "DDBJ.wsdl"
-
-    # returns a Bio::DDBJ::XML::DDBJ object.
-    def initialize(wsdl = nil)
-      super(wsdl || SERVER_URI)
-    end
   end
 
 
@@ -196,8 +170,8 @@ class XML < Bio::SOAPWSDL
   #
   # === WSDL Methods
   #
-  # ==== searchSimple(program, database, query)
-  # ==== searchParam(program, database, query, param)
+  # * searchSimple(program, database, query)
+  # * searchParam(program, database, query, param)
   #
   # === References
   #
@@ -205,11 +179,6 @@ class XML < Bio::SOAPWSDL
   #
   class Fasta < XML
     SERVER_URI = BASE_URI + "Fasta.wsdl"
-
-    # returns a Bio::DDBJ::XML::Fasta object.
-    def initialize(wsdl = nil)
-      super(wsdl || SERVER_URI)
-    end
   end
 
 
@@ -227,35 +196,35 @@ class XML < Bio::SOAPWSDL
   #
   # === WSDL Methods
   #
-  # ==== getEntry(database, var, param1, param2)
-  # ==== getEntry(database, var)
-  # ==== getDDBJEntry(accession)
-  # ==== getDDBJCONEntry(accession)
-  # ==== getDDBJVerEntry(accession)
-  # ==== getLocus_DDBJEntry(locus)
-  # ==== getGene_DDBJEntry(gene)
-  # ==== getProd_DDBJEntry(products)
-  # ==== getPID_DDBJEntry(pid)
-  # ==== getClone_DDBJEntry(clone)
-  # ==== getXML_DDBJEntry(accession)
-  # ==== getEMBLEntry(accession)
-  # ==== getSWISSEntry(accession)
-  # ==== getPIREntry(accession)
-  # ==== getPRFEntry(accession)
-  # ==== getPDBEntry(accession)
-  # ==== getQVEntry(accession)
-  # ==== getDADEntry(accession)
-  # ==== getPID_DADEntry(pid)
-  # ==== getFASTA_DDBJEntry(accession)
-  # ==== getFASTA_DDBJCONEntry(accession)
-  # ==== getFASTA_DDBJVerEntry(accession)
-  # ==== getFASTA_DDBJSeqEntry(accession, start, end)
-  # ==== getFASTA_DADEntry(accession)
-  # ==== getFASTA_PIREntry(accession)
-  # ==== getFASTA_SWISSEntry(accession)
-  # ==== getFASTA_PDBEntry(accession)
-  # ==== getFASTA_PRFEntry(accession)
-  # ==== getFASTA_CDSEntry(accession)
+  # * getEntry(database, var, param1, param2)
+  # * getEntry(database, var)
+  # * getDDBJEntry(accession)
+  # * getDDBJCONEntry(accession)
+  # * getDDBJVerEntry(accession)
+  # * getLocus_DDBJEntry(locus)
+  # * getGene_DDBJEntry(gene)
+  # * getProd_DDBJEntry(products)
+  # * getPID_DDBJEntry(pid)
+  # * getClone_DDBJEntry(clone)
+  # * getXML_DDBJEntry(accession)
+  # * getEMBLEntry(accession)
+  # * getSWISSEntry(accession)
+  # * getPIREntry(accession)
+  # * getPRFEntry(accession)
+  # * getPDBEntry(accession)
+  # * getQVEntry(accession)
+  # * getDADEntry(accession)
+  # * getPID_DADEntry(pid)
+  # * getFASTA_DDBJEntry(accession)
+  # * getFASTA_DDBJCONEntry(accession)
+  # * getFASTA_DDBJVerEntry(accession)
+  # * getFASTA_DDBJSeqEntry(accession, start, end)
+  # * getFASTA_DADEntry(accession)
+  # * getFASTA_PIREntry(accession)
+  # * getFASTA_SWISSEntry(accession)
+  # * getFASTA_PDBEntry(accession)
+  # * getFASTA_PRFEntry(accession)
+  # * getFASTA_CDSEntry(accession)
   #
   # === References
   #
@@ -263,11 +232,6 @@ class XML < Bio::SOAPWSDL
   #
   class GetEntry < XML
     SERVER_URI = BASE_URI + "GetEntry.wsdl"
-
-    # returns a Bio::DDBJ::XML::GetEntry object.
-    def initialize(wsdl = nil)
-      super(wsdl || SERVER_URI)
-    end
   end
 
 
@@ -294,17 +258,17 @@ class XML < Bio::SOAPWSDL
   #
   # === WSDL Methods
   #
-  # ==== getOrganismList
-  # ==== getChIDList
-  # ==== getOrganismNameFromChid(chid)
-  # ==== getChIDFromOrganismName(orgName)
-  # ==== getAccession(chid)
-  # ==== getPieceNumber(chid)
-  # ==== getDivision(chid)
-  # ==== getType(chid)
-  # ==== getFlatFile(chid)
-  # ==== getFastaFile(chid, type)
-  # ==== getCDS(chid)
+  # * getOrganismList
+  # * getChIDList
+  # * getOrganismNameFromChid(chid)
+  # * getChIDFromOrganismName(orgName)
+  # * getAccession(chid)
+  # * getPieceNumber(chid)
+  # * getDivision(chid)
+  # * getType(chid)
+  # * getFlatFile(chid)
+  # * getFastaFile(chid, type)
+  # * getCDS(chid)
   #
   # === References
   #
@@ -312,11 +276,6 @@ class XML < Bio::SOAPWSDL
   #
   class Gib < XML
     SERVER_URI = BASE_URI + "Gib.wsdl"
-
-    # returns a Bio::DDBJ::XML::Gib object.
-    def initialize(wsdl = nil)
-      super(wsdl || SERVER_URI)
-    end
   end
 
   
@@ -334,8 +293,8 @@ class XML < Bio::SOAPWSDL
   #
   # === WSDL Methods
   #
-  # ==== getOrganismList
-  # ==== getMasterInfo(orfID, organism)
+  # * getOrganismList
+  # * getMasterInfo(orfID, organism)
   #
   # === References
   #
@@ -343,11 +302,6 @@ class XML < Bio::SOAPWSDL
   #
   class Gtop < XML
     SERVER_URI = BASE_URI + "Gtop.wsdl"
-
-    # returns a Bio::DDBJ::XML::Gtop object.
-    def initialize(wsdl = nil)
-      super(wsdl || SERVER_URI)
-    end
   end
 
 
@@ -364,12 +318,12 @@ class XML < Bio::SOAPWSDL
   #
   # === WSDL Methods
   #
-  # ==== searchVariation(field, query, order)
-  # ==== searchVariationSimple(field, query)
-  # ==== searchFrequency(field, query, order)
-  # ==== searchFrequencySimple(field, query)
-  # ==== getVariation(variation_id)
-  # ==== getFrequency(variation_id, population_id)
+  # * searchVariation(field, query, order)
+  # * searchVariationSimple(field, query)
+  # * searchFrequency(field, query, order)
+  # * searchFrequencySimple(field, query)
+  # * getVariation(variation_id)
+  # * getFrequency(variation_id, population_id)
   #
   # === References
   #
@@ -377,11 +331,6 @@ class XML < Bio::SOAPWSDL
   #
   class PML < XML
     SERVER_URI = BASE_URI + "PML.wsdl"
-
-    # returns a Bio::DDBJ::XML::PML object.
-    def initialize(wsdl = nil)
-      super(wsdl || SERVER_URI)
-    end
   end
 
 
@@ -399,8 +348,8 @@ class XML < Bio::SOAPWSDL
   #
   # === WSDL Methods
   #
-  # ==== searchSimple(query)
-  # ==== searchParam(query, param)
+  # * searchSimple(query)
+  # * searchParam(query, param)
   #
   # === Examples
   #
@@ -408,11 +357,6 @@ class XML < Bio::SOAPWSDL
   #
   class SRS < XML
     SERVER_URI = BASE_URI + "SRS.wsdl"
-
-    # returns a Bio::DDBJ::XML::SRS object.
-    def initialize(wsdl = nil)
-      super(wsdl || SERVER_URI)
-    end
   end
   
 
@@ -436,11 +380,11 @@ class XML < Bio::SOAPWSDL
   #
   # === WSDL Methdos
   #
-  # ==== searchSimple(tx_Name)
-  # ==== searchParam(tx_Name, tx_Clas, tx_Rank, tx_Rmax, tx_Dcls)
-  # ==== getTxId(tx_Name)
-  # ==== getTxName(tx_Id)
-  # ==== searchLineage(query, ranks, superkingdom)
+  # * searchSimple(tx_Name)
+  # * searchParam(tx_Name, tx_Clas, tx_Rank, tx_Rmax, tx_Dcls)
+  # * getTxId(tx_Name)
+  # * getTxName(tx_Id)
+  # * searchLineage(query, ranks, superkingdom)
   # 
   # === References
   #
@@ -448,11 +392,6 @@ class XML < Bio::SOAPWSDL
   #
   class TxSearch < XML
     SERVER_URI = BASE_URI + "TxSearch.wsdl"
-
-    # returns a Bio::DDBJ::XML::TxSearch object.
-    def initialize(wsdl = nil)
-      super(wsdl || SERVER_URI)
-    end
   end
 
 end # XML
