@@ -3,9 +3,9 @@
 #
 #   Copyright (C) 2001-2006 Naohisa Goto <ng@bioruby.org>
 #
-# License:: Ruby's
+# License:: The Ruby License
 #
-#  $Id: flatfile.rb,v 1.46 2006/02/22 10:01:27 ngoto Exp $
+#  $Id: flatfile.rb,v 1.60 2007/07/09 14:08:34 ngoto Exp $
 #
 #
 # Bio::FlatFile is a helper and wrapper class to read a biological data file.
@@ -34,7 +34,6 @@ module Bio
         @path = path
         # initialize prefetch buffer
         @buffer = ''
-        @path = path
       end
 
       # Creates a new input stream wrapper from the given IO object.
@@ -262,6 +261,9 @@ module Bio
         # the last entry read from the stream
         attr_reader :entry
 
+        # a flag to write down entry start and end positions
+        attr_accessor :entry_pos_flag
+
         # start position of the entry
         attr_reader :entry_start_pos
 
@@ -290,6 +292,7 @@ module Bio
             end
           end
           @delimiter_overrun = klass::DELIMITER_OVERRUN rescue nil
+          @entry_pos_flag = nil
         end
 
         # (String) delimiter indicates the end of a entry.
@@ -330,7 +333,7 @@ module Bio
 
         # gets a entry
         def get_entry
-          p0 = @stream.pos
+          p0 = @entry_pos_flag ? @stream.pos : nil
           e  = @stream.gets(@delimiter)
           if e and @delimiter_overrun then
             if e[-@delimiter.size, @delimiter.size ] == @delimiter then
@@ -339,7 +342,7 @@ module Bio
               @stream.ungets(overrun)
             end
           end
-          p1 = @stream.pos
+          p1 = @entry_pos_flag ? @stream.pos : nil
           @entry_start_pos = p0
           @entry = e
           @entry_ended_pos = p1
@@ -422,7 +425,7 @@ module Bio
       # check if file is filename or IO object
       unless file.respond_to?(:gets)
         # 'file' is a filename
-        self.open_file(file, *arg, &block)
+        _open_file(dbclass, file, *arg, &block)
       else
         # 'file' is a IO object
         ff = self.new(dbclass, file)
@@ -462,15 +465,27 @@ module Bio
     # Otherwise, it returns a new FlatFile object.
     #
     def self.open_file(filename, *arg)
+      _open_file(nil, filename, *arg)
+    end
+
+    # Same as FlatFile.open(dbclass, filename, *arg),
+    # except that it only accept filename and doesn't accept IO object.
+    #
+    # It can accept a block.
+    # If a block is given, it returns the block's return value.
+    # Otherwise, it returns a new FlatFile object.
+    #
+    def self._open_file(dbclass, filename, *arg)
       if block_given? then
         BufferedInputStream.open_file(filename, *arg) do |stream|
-          yield self.new(nil, stream)
+          yield self.new(dbclass, stream)
         end
       else
         stream = BufferedInputStream.open_file(filename, *arg)
-        self.new(nil, stream)
+        self.new(dbclass, stream)
       end
     end
+    private_class_method :_open_file
 
     # Opens URI specified as _uri_.
     # _uri_ must be a String or URI object.
@@ -493,6 +508,20 @@ module Bio
       end
     end
 
+    # Executes the block for every entry in the stream.
+    # Same as FlatFile.open(*arg) { |ff| ff.each { |entry| ... }}.
+    # 
+    # * Example
+    #     Bio::FlatFile.foreach('test.fst') { |e| puts e.definition }
+    #
+    def self.foreach(*arg)
+      self.open(*arg) do |flatfileobj|
+        flatfileobj.each do |entry|
+          yield entry
+        end
+      end
+    end
+
     # Same as FlatFile.open, except that 'stream' should be a opened
     # stream object (IO, File, ..., who have the 'gets' method).
     #
@@ -518,13 +547,11 @@ module Bio
     #
     def initialize(dbclass, stream)
       # 2nd arg: IO object
-      if @stream.kind_of?(BufferedInputStream)
+      if stream.kind_of?(BufferedInputStream)
         @stream = stream
       else
         @stream = BufferedInputStream.for_io(stream)
       end
-      # default is raw mode
-      self.raw = false
       # 1st arg: database class (or file format autodetection)
       if dbclass then
 	self.dbclass = dbclass
@@ -534,6 +561,8 @@ module Bio
       #
       @skip_leader_mode = :firsttime
       @firsttime_flag = true
+      # default raw mode is false
+      self.raw = false
     end
 
     # The mode how to skip leader of the data.
@@ -563,8 +592,14 @@ module Bio
       @stream.path
     end
 
+    # Exception class to be raised when data format hasn't been specified.
+    class UnknownDataFormatError < IOError
+    end
+
     # Get next entry.
     def next_entry
+      raise UnknownDataFormatError, 
+      'file format auto-detection failed?' unless @dbclass
       if @skip_leader_mode and
           ((@firsttime_flag and @skip_leader_mode == :firsttime) or
              @skip_leader_mode == :everytime)
@@ -587,6 +622,16 @@ module Bio
       @splitter.entry
     end
 
+    # a flag to write down entry start and end positions
+    def entry_pos_flag
+      @splitter.entry_pos_flag
+    end
+
+    # Sets flag to write down entry start and end positions
+    def entry_pos_flag=(x)
+      @splitter.entry_pos_flag = x
+    end
+
     # start position of the last entry
     def entry_start_pos
       @splitter.entry_start_pos
@@ -736,21 +781,26 @@ module Bio
 
       include TSort
 
+      # Array to store autodetection rules.
+      # This is defined only for inspect.
+      class RulesArray < Array
+        # visualize contents
+        def inspect
+          "[#{self.collect { |e| e.name.inspect }.join(' ')}]"
+        end
+      end #class RulesArray
+
       # Template of a single rule of autodetection
       class RuleTemplate
         # Creates a new element.
         def self.[](*arg)
           self.new(*arg)
         end
-          
+        
         # Creates a new element.
         def initialize
-          a = Array.new
-          def a.inspect
-            "[#{self.collect { |e| e.name.inspect }.join(' ')}]"
-          end
-          @higher_priority_elements = a.clone
-          @lower_priority_elements  = a.clone
+          @higher_priority_elements = RulesArray.new
+          @lower_priority_elements  = RulesArray.new
           @name = nil
         end
 
@@ -784,6 +834,24 @@ module Bio
         def guess(text, meta)
           nil
         end
+
+        private
+        # Gets constant from constant name given as a string.
+        def str2const(str)
+          const = Object
+          str.split(/\:\:/).each do |x|
+            const = const.const_get(x)
+          end
+          const
+        end
+
+        # Gets database class from given object.
+        # Current implementation is: 
+        # if _obj_ is kind of String, regarded as a constant.
+        # Otherwise, returns _obj_ as is.
+        def get_dbclass(obj)
+          obj.kind_of?(String) ? str2const(obj) : obj
+        end
       end #class Rule_Template
 
       # RuleDebug is a class for debugging autodetect classes/methods
@@ -835,40 +903,49 @@ module Bio
         def initialize(dbclass, re)
           super()
           @re = re
-          @dbclass = dbclass
-          @dbclasses = [ dbclass ]
           @name = dbclass.to_s
+          @dbclass = nil
+          @dbclass_lazy = dbclass
+        end
+
+        # database class (lazy evaluation)
+        def dbclass
+          unless @dbclass
+            @dbclass = get_dbclass(@dbclass_lazy)
+          end
+          @dbclass
+        end
+        private :dbclass
+
+        # returns database classes
+        def dbclasses
+          [ dbclass ]
         end
 
         # If given text matches the regexp, returns the database class.
         # Otherwise, returns nil or false.
         # _meta_ is ignored.
         def guess(text, meta)
-          @re =~ text ? @dbclass : nil
+          @re =~ text ? dbclass : nil
         end
       end #class RuleRegexp
 
       # A autodetection rule to use more than two regular expressions.
-      class RuleRegexp2 < RuleTemplate
+      # If given string matches one of the regular expressions,
+      # returns the database class.
+      class RuleRegexp2 < RuleRegexp
         # Creates a new instance.
         def initialize(dbclass, *regexps)
-          super()
+          super(dbclass, nil)
           @regexps = regexps
-          @dbclass = dbclass
-          @dbclasses = [ dbclass ]
-          if name
-            @name = name
-          else
-            @name = @dbclass.to_s
-          end
         end
 
-        # If given text matches the regexp, returns the database class.
+        # If given text matches one of the regexp, returns the database class.
         # Otherwise, returns nil or false.
         # _meta_ is ignored.
         def guess(text, meta)
           @regexps.each do |re|
-            return @dbclass if re =~ text
+            return dbclass if re =~ text
           end
           nil
         end
@@ -880,10 +957,19 @@ module Bio
         def initialize(*dbclasses, &proc)
           super()
           @proc = proc
-          @dbclasses = dbclasses
+          @dbclasses = nil
+          @dbclasses_lazy = dbclasses
           @name = dbclasses.collect { |x| x.to_s }.join('|')
         end
 
+        # database classes (lazy evaluation)
+        def dbclasses
+          unless @dbclasses
+            @dbclasses = @dbclasses_lazy.collect { |x| get_dbclass(x) }
+          end
+          @dbclasses
+        end
+
         # If given text (and/or meta information) is known, returns
         # the database class.
         # Otherwise, returns nil or false.
@@ -1039,22 +1125,23 @@ module Bio
       # make a default of default autodetect object
       def self.make_default
         a = self[
-          genbank  = RuleRegexp[ Bio::GenBank,
+          genbank  = RuleRegexp[ 'Bio::GenBank',
             /^LOCUS       .+ bp .*[a-z]*[DR]?NA/ ],
-          genpept  = RuleRegexp[ Bio::GenPept,
+          genpept  = RuleRegexp[ 'Bio::GenPept',
             /^LOCUS       .+ aa .+/ ],
-          medline  = RuleRegexp[ Bio::MEDLINE,
+          medline  = RuleRegexp[ 'Bio::MEDLINE',
             /^UI  \- [0-9]+$/ ],
-          embl     = RuleRegexp[ Bio::EMBL,
+          embl     = RuleRegexp[ 'Bio::EMBL',
             /^ID   .+\; .*(DNA|RNA|XXX)\;/ ],
-          sptr     = RuleRegexp[ Bio::SPTR,
-            /^ID   .+\; *PRT\;/ ],
-          prosite  = RuleRegexp[ Bio::PROSITE,
+          sptr     = RuleRegexp2[ 'Bio::SPTR',
+            /^ID   .+\; *PRT\;/,
+            /^ID   [-A-Za-z0-9_\.]+ .+\; *[0-9]+ *AA\./ ],
+          prosite  = RuleRegexp[ 'Bio::PROSITE',
             /^ID   [-A-Za-z0-9_\.]+\; (PATTERN|RULE|MATRIX)\.$/ ],
-          transfac = RuleRegexp[ Bio::TRANSFAC,
+          transfac = RuleRegexp[ 'Bio::TRANSFAC',
             /^AC  [-A-Za-z0-9_\.]+$/ ],
 
-          aaindex  = RuleProc.new(Bio::AAindex1, Bio::AAindex2) do |text|
+          aaindex  = RuleProc.new('Bio::AAindex1', 'Bio::AAindex2') do |text|
             if /^H [-A-Z0-9_\.]+$/ =~ text then
               if text =~ /^M [rc]/ then
                 Bio::AAindex2
@@ -1068,33 +1155,35 @@ module Bio
             end
           end,
 
-          litdb    = RuleRegexp[ Bio::LITDB,
+          litdb    = RuleRegexp[ 'Bio::LITDB',
             /^CODE        [0-9]+$/ ],
-          brite    = RuleRegexp[ Bio::KEGG::BRITE,
+          brite    = RuleRegexp[ 'Bio::KEGG::BRITE',
             /^Entry           [A-Z0-9]+/ ],
-          ko       = RuleRegexp[ Bio::KEGG::KO,
+          orthology = RuleRegexp[ 'Bio::KEGG::ORTHOLOGY',
             /^ENTRY       .+ KO\s*/ ],
-          glycan   = RuleRegexp[ Bio::KEGG::GLYCAN,
+          drug     = RuleRegexp[ 'Bio::KEGG::DRUG',
+            /^ENTRY       .+ Drug\s*/ ],
+          glycan   = RuleRegexp[ 'Bio::KEGG::GLYCAN',
             /^ENTRY       .+ Glycan\s*/ ],
-          enzyme   = RuleRegexp2[ Bio::KEGG::ENZYME,
+          enzyme   = RuleRegexp2[ 'Bio::KEGG::ENZYME',
             /^ENTRY       EC [0-9\.]+$/,
             /^ENTRY       .+ Enzyme\s*/
           ],
-          compound = RuleRegexp2[ Bio::KEGG::COMPOUND,
+          compound = RuleRegexp2[ 'Bio::KEGG::COMPOUND',
             /^ENTRY       C[A-Za-z0-9\._]+$/,
             /^ENTRY       .+ Compound\s*/
           ],
-          reaction = RuleRegexp2[ Bio::KEGG::REACTION,
+          reaction = RuleRegexp2[ 'Bio::KEGG::REACTION',
             /^ENTRY       R[A-Za-z0-9\._]+$/,
             /^ENTRY       .+ Reaction\s*/
           ],
-          genes    = RuleRegexp[ Bio::KEGG::GENES,
-            /^ENTRY       .+ (CDS|gene|.*RNA) / ],
-          genome   = RuleRegexp[ Bio::KEGG::GENOME,
+          genes    = RuleRegexp[ 'Bio::KEGG::GENES',
+            /^ENTRY       .+ (CDS|gene|.*RNA|Contig) / ],
+          genome   = RuleRegexp[ 'Bio::KEGG::GENOME',
             /^ENTRY       [a-z]+$/ ],
 
-          fantom = RuleProc.new(Bio::FANTOM::MaXML::Cluster,
-                                Bio::FANTOM::MaXML::Sequence) do |text|
+          fantom = RuleProc.new('Bio::FANTOM::MaXML::Cluster',
+                                'Bio::FANTOM::MaXML::Sequence') do |text|
             if /\<\!DOCTYPE\s+maxml\-(sequences|clusters)\s+SYSTEM/ =~ text
               case $1
               when 'clusters'
@@ -1109,37 +1198,44 @@ module Bio
             end
           end,
 
-          pdb = RuleRegexp[ Bio::PDB,
+          pdb = RuleRegexp[ 'Bio::PDB',
             /^HEADER    .{40}\d\d\-[A-Z]{3}\-\d\d   [0-9A-Z]{4}/ ],
-          het = RuleRegexp[ Bio::PDB::ChemicalComponent,
+          het = RuleRegexp[ 'Bio::PDB::ChemicalComponent',
             /^RESIDUE +.+ +\d+\s*$/ ],
 
-          clustal = RuleRegexp[ Bio::ClustalW::Report,
-          /^CLUSTAL .*\(.*\).*sequence +alignment/ ],
+          clustal = RuleRegexp2[ 'Bio::ClustalW::Report',
+          /^CLUSTAL .*\(.*\).*sequence +alignment/,
+          /^CLUSTAL FORMAT for T-COFFEE/ ],
 
-          blastxml = RuleRegexp[ Bio::Blast::Report,
+          gcg_msf = RuleRegexp[ 'Bio::GCG::Msf',
+          /^!!(N|A)A_MULTIPLE_ALIGNMENT .+/ ],
+
+          gcg_seq = RuleRegexp[ 'Bio::GCG::Seq',
+          /^!!(N|A)A_SEQUENCE .+/ ],
+
+          blastxml = RuleRegexp[ 'Bio::Blast::Report',
             /\<\!DOCTYPE BlastOutput PUBLIC / ],
-          wublast  = RuleRegexp[ Bio::Blast::WU::Report,
+          wublast  = RuleRegexp[ 'Bio::Blast::WU::Report',
             /^BLAST.? +[\-\.\w]+\-WashU +\[[\-\.\w ]+\]/ ],
-          wutblast = RuleRegexp[ Bio::Blast::WU::Report_TBlast,
+          wutblast = RuleRegexp[ 'Bio::Blast::WU::Report_TBlast',
             /^TBLAST.? +[\-\.\w]+\-WashU +\[[\-\.\w ]+\]/ ],
-          blast    = RuleRegexp[ Bio::Blast::Default::Report,
+          blast    = RuleRegexp[ 'Bio::Blast::Default::Report',
             /^BLAST.? +[\-\.\w]+ +\[[\-\.\w ]+\]/ ],
-          tblast   = RuleRegexp[ Bio::Blast::Default::Report_TBlast,
+          tblast   = RuleRegexp[ 'Bio::Blast::Default::Report_TBlast',
             /^TBLAST.? +[\-\.\w]+ +\[[\-\.\w ]+\]/ ],
 
-          blat   = RuleRegexp[ Bio::Blat::Report,
-            /^psLayout version \d+\s*$/ ],
-          spidey = RuleRegexp[ Bio::Spidey::Report,
+          blat   = RuleRegexp[ 'Bio::Blat::Report',
+            /^psLayout version \d+/ ],
+          spidey = RuleRegexp[ 'Bio::Spidey::Report',
             /^\-\-SPIDEY version .+\-\-$/ ],
-          hmmer  = RuleRegexp[ Bio::HMMER::Report,
+          hmmer  = RuleRegexp[ 'Bio::HMMER::Report',
             /^HMMER +\d+\./ ],
-          sim4   = RuleRegexp[ Bio::Sim4::Report,
+          sim4   = RuleRegexp[ 'Bio::Sim4::Report',
             /^seq1 \= .*\, \d+ bp(\r|\r?\n)seq2 \= .*\, \d+ bp(\r|\r?\n)/ ],
 
-          fastaformat = RuleProc.new(Bio::FastaFormat,
-                                     Bio::NBRF,
-                                     Bio::FastaNumericFormat) do |text|
+          fastaformat = RuleProc.new('Bio::FastaFormat',
+                                     'Bio::NBRF',
+                                     'Bio::FastaNumericFormat') do |text|
             if /^>.+$/ =~ text
               case text
               when /^>([PF]1|[DR][LC]|N[13]|XX)\;.+/
@@ -1167,8 +1263,9 @@ module Bio
         # KEGG
         #aaindex.is_prior_to litdb
         #litdb.is_prior_to brite
-        brite.is_prior_to ko
-        ko.is_prior_to glycan
+        brite.is_prior_to orthology
+        orthology.is_prior_to drug
+        drug.is_prior_to glycan
         glycan.is_prior_to enzyme
         enzyme.is_prior_to compound
         compound.is_prior_to reaction
