@@ -3,9 +3,9 @@
 #
 # Copyright::   Copyright (C) 2002, 2003, 2004, 2005
 #               Toshiaki Katayama <k@bioruby.org>
-# License::     LGPL
+# License::     The Ruby License
 #
-# $Id: registry.rb,v 1.16 2005/12/18 15:58:42 k Exp $
+# $Id: registry.rb,v 1.19 2007/04/05 23:35:41 trevor Exp $
 #
 # == Description
 #
@@ -81,26 +81,10 @@
 # * http://cvs.open-bio.org/cgi-bin/viewcvs/viewcvs.cgi/obda-specs/?cvsroot=obf-common
 # * http://www.open-bio.org/registry/seqdatabase.ini
 #
-#--
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Lesser General Public
-#  License as published by the Free Software Foundation; either
-#  version 2 of the License, or (at your option) any later version.
-#
-#  This library is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Lesser General Public License for more details.
-#
-#  You should have received a copy of the GNU Lesser General Public
-#  License along with this library; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-#
-#++
-#
 
 require 'uri'
 require 'net/http'
+require 'bio/command'
 
 
 module Bio
@@ -187,7 +171,7 @@ class Registry
 
   def read_remote(url)
     schema, user, host, port, reg, path, = URI.split(url)
-    Net::HTTP.start(host, port) do |http|
+    Bio::Command.start_http(host, port) do |http|
       response, = http.get(path)
       parse_stanza(response.body)
     end
