@@ -3,9 +3,9 @@
 #
 # Copyright::   Copyright (C) 2006
 #               Toshiaki Katayama <k@bioruby.org>
-# License::     Ruby's
+# License::     The Ruby License
 #
-# $Id: generic.rb,v 1.3 2006/02/06 14:26:04 k Exp $
+# $Id: generic.rb,v 1.5 2007/04/05 23:35:41 trevor Exp $
 #
 
 require 'bio/sequence/common'
@@ -13,7 +13,7 @@ require 'bio/sequence/common'
 module Bio
 class Sequence
 
-class Generic < String
+class Generic < String #:nodoc:
 
   include Bio::Sequence::Common
 
