@@ -3,22 +3,26 @@
 #
 # Copyright::   Copyright (C) 2005, 2006
 #               Toshiaki Katayama <k@bioruby.org>
-# License:      Ruby's
+# License::     The Ruby License
 #
-# $Id: shell.rb,v 1.15 2006/02/27 09:34:23 k Exp $
+# $Id: shell.rb,v 1.20 2007/07/09 11:17:09 k Exp $
 #
 
 require 'bio'
 require 'yaml'
 require 'open-uri'
+require 'fileutils'
 require 'pp'
 
 module Bio::Shell
 
+  require 'bio/shell/setup'
+  require 'bio/shell/irb'
+  require 'bio/shell/web'
+  require 'bio/shell/script'
   require 'bio/shell/core'
   require 'bio/shell/interface'
   require 'bio/shell/object'
-  require 'bio/shell/web'
   require 'bio/shell/demo'
   require 'bio/shell/plugin/entry'
   require 'bio/shell/plugin/seq'
@@ -26,11 +30,14 @@ module Bio::Shell
   require 'bio/shell/plugin/codon'
   require 'bio/shell/plugin/flatfile'
   require 'bio/shell/plugin/obda'
+  require 'bio/shell/plugin/das'
   require 'bio/shell/plugin/keggapi'
+  require 'bio/shell/plugin/soap'
   require 'bio/shell/plugin/emboss'
+  require 'bio/shell/plugin/blast'
+  require 'bio/shell/plugin/psort'
 
   extend Ghost
-  extend Private
 
 end
 
