@@ -3,9 +3,9 @@
 #
 # Copyright::   Copyright (C) 2004, 2005
 #               Itoshi NIKAIDO <dritoshi@gmail.com>
-# License::     LGPL
+# License::     The Ruby License
 #
-# $Id: sirna.rb,v 1.7 2005/12/27 17:27:38 k Exp $
+# $Id: sirna.rb,v 1.11 2007/04/05 23:35:41 trevor Exp $
 #
 # == Bio::SiRNA - Designing siRNA.
 #
@@ -38,24 +38,6 @@
 # * Angela Reynolds et al.  Rational siRNA design for RNA interference.
 #   Nature Biotech. 2004 22: 326-330.
 #
-#--
-#
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Lesser General Public
-#  License as published by the Free Software Foundation; either
-#  version 2 of the License, or (at your option) any later version.
-#
-#  This library is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Lesser General Public License for more details.
-#
-#  You should have received a copy of the GNU Lesser General Public
-#  License along with this library; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-#
-#++
-#
 
 require 'bio/sequence'
 
@@ -96,8 +78,8 @@ module Bio
       one_third  = target.size * 1 / 3
       start_pos  = @target_size - one_third - 1
       remain_seq = target.subseq(start_pos, @target_size - 2)
-      gc_number  = remain_seq.scan(/[AU]/i).size
-      return false if gc_number < 5
+      au_number  = remain_seq.scan(/[AU]/i).size
+      return false if au_number < 5
   
       return true
     end
