@@ -1,23 +1,10 @@
 #
 # test/unit/bio/db/test_prosite.rb - Unit test for Bio::PROSITE
 #
-#   Copyright (C) 2005 Mitsuteru Nakao <n@bioruby.org>
+# Copyright::  Copyright (C) 2005 Mitsuteru Nakao <n@bioruby.org>
+# License::    The Ruby License
 #
-#  This library is free software; you can redistribute it and/or
-#  modify it under the terms of the GNU Lesser General Public
-#  License as published by the Free Software Foundation; either
-#  version 2 of the License, or (at your option) any later version.
-#
-#  This library is distributed in the hope that it will be useful,
-#  but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  Lesser General Public License for more details.
-#
-#  You should have received a copy of the GNU Lesser General Public
-#  License along with this library; if not, write to the Free Software
-#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
-#
-#  $Id: test_prosite.rb,v 1.3 2005/12/18 18:21:58 k Exp $
+#  $Id: test_prosite.rb,v 1.6 2007/04/05 23:35:43 trevor Exp $
 #
 
 require 'pathname'
@@ -1438,12 +1425,12 @@ module Bio
 
     def test_pa2re
       pa = '[AC]-x-V-x(4)-{ED}.'
-      assert_equal(/[AC].V.{4}[^ED]/, @obj.pa2re(pa))
+      assert_equal(/[AC].V.{4}[^ED]/i, @obj.pa2re(pa))
     end
 
     def test_self_pa2re
       pa = '[AC]-x-V-x(4)-{ED}.'
-      assert_equal(/[AC].V.{4}[^ED]/, Bio::PROSITE.pa2re(pa))
+      assert_equal(/[AC].V.{4}[^ED]/i, Bio::PROSITE.pa2re(pa))
     end
 
   end # class TestPROSITE
